#!/usr/bin/env python

"""
synopsis:
    Read CSV file and print report, then write out modified data
    to new CSV file.
usage:
    python csv01.py <in_file_name> <out_file_name>
"""

from __future__ import print_function
import sys
import csv


def print_report(infilename):
    with open(infilename, 'r') as csvfile:
        csvreader = csv.reader(csvfile)
        for row in csvreader:
            line = format_line(row)
            print(line)


def format_line(row):
    field1, field2, field3, field4 = row
    col1 = field1.ljust(30)
    col2 = field2.rjust(10)
    col3 = field3.ljust(20)
    col4 = field4.rjust(10)
    line = '{}  {}  {}  {}'.format(col1, col2, col3, col4)
    return line


def write_update(infilename, outfilename):
    with open(infilename, 'r') as csvfile:
        csvreader = csv.reader(csvfile)
        with open(outfilename, 'w') as csvoutfile:
            csvwriter = csv.writer(csvoutfile)
            for count, row in enumerate(csvreader):
                new_row = transform_row(row, count)
                csvwriter.writerow(new_row)


def transform_row(row, count):
    field1, field2, field3, field4 = row
    field1a = '{}, "{}"'.format(count, field1)
    field2a = '{}'.format(int(field2) * (count + 4))
    field3a = '{}. {}'.format(count, field3)
    field4a = '{}'.format(int(field4) * (count + 2))
    new_row = (field1a, field2a, field3a, field4a)
    return new_row


def main():
    args = sys.argv[1:]
    if len(args) != 2:
        sys.exit(__doc__)
    infilename = args[0]
    outfilename = args[1]
    print('infilename:', infilename)
    print_report(infilename)
    write_update(infilename, outfilename)


if __name__ == '__main__':
    main()
