#!/usr/bin/env python

"""
synopsis:
    Serve documents from a document database.
    Use ZeroMQ as a transport layer.
    Support the following API:
    - add -- add a new document.
    - get - retrieve document by ID.
    - search -- retrieve list of (ID, tags, description) for documents
      with the given tags.
    - list -- retrieve a list of all documents
usage:
    The following runs a simple test:
        python doc_database_lib.py <database_file.sqlite>
example:
    python doc_database_lib.py my_database_file.sqlite
"""

import sys
import sqlite3


class DocDatabase(object):
    def __init__(self):
        self.db_name = None
        self.connection = None
        self.cursor = None

    def connect(self, db_name):
        """Create and save a connection to the database (file)."""
        self.db_name = db_name
        self.connection = sqlite3.connect(self.db_name)
        self.cursor = self.connection.cursor()

    def close(self):
        """Close the database connection."""
        self.connection.close()
        self.connection = None
        self.cursor = None

    def commit(self):
        """Commit changes to the database."""
        self.connection.commit()
    flush = commit

    def list(self):
        """Return a list of all the documents in the database.
        Each list item is a list containing: [id, tags, description].
        """
        sql = 'select id, tags,description from documents'
        self.cursor.execute(sql)
        result_block = [
            [id, tags, description] for (id, tags, description) in self.cursor]
        return result_block

    def get(self, id):
        """Retrieve a document by ID and return it."""
        sql = 'select * from documents where id = {}'.format(id)
        self.cursor.execute(sql)
        rows = self.cursor.fetchall()
        if len(rows) < 1:
            result_block = None
        else:
            result_block = rows[0]
        return result_block

    def search(self, tag):
        """Retrieve and return a list of all documents that have a tag.
        Each list item is a list containing: [id, tags, description].
        """
        sql = 'select id, tags, description from documents'
        self.cursor.execute(sql)
        result_block = []
        for (id, tags, description) in self.cursor:
            tag_list = tags.split()
            if tag in tag_list:
                result_block.append([id, tags, description])
        return result_block

    def add(self, tags, description, body):
        """Add a document to the database given its tags, description,
        and body.
        """
        body = body.replace('"', '')
        sql = 'select latest_id from config'
        self.cursor.execute(sql)
        row = self.cursor.fetchall()
        id = row[0][0]
        new_id = id + 1
        self.cursor.execute('update config set latest_id={}'.format(new_id))
        sql = 'insert into documents values ({}, "{}", "{}", "{}")'.format(
            new_id, tags, description, body)
        self.cursor.execute(sql)
        self.connection.commit()
        result = {
            'flag': 'success',
            'id': new_id,
        }
        return result


def usage():
    sys.exit(__doc__)


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        usage()
    db_name = args[0]
    database = DocDatabase()
    database.connect(db_name)
    docs = database.list()
    for doc in docs:
        print 'doc: {}'.format(doc)
    id = docs[0][0]
    doc = database.get(id)
    print '-' * 60
    print 'doc: {}'.format(doc)


if __name__ == '__main__':
    import pdb; pdb.set_trace()
    main()
