#!/usr/bin/env python

"""
synopsis:
    Client that requests document services.
    Use ZeroMQ as a transport layer.
    Request the following services:
    - add document.
    - list all documents in the database.
    - get document by ID.
    - search for documents with a given tag.
      given tags.
usage:
    python zmq_doc_client01.py [options] command arg1 arg2 ...
options:
    None
examples:
    python zmq_doc_client01.py add "tag tag" "description" <filename>
    python zmq_doc_client01.py list
    python zmq_doc_client01.py get <id>
    python zmq_doc_client01.py search <tag>
"""


import sys
import getopt
import json
import zmq


def add_request(args, show=True):
    body = None
    infilename = args[2]
    with open(infilename, 'r') as infile:
        body = infile.read()
    context = zmq.Context()
    socket = context.socket(zmq.REQ)
    socket.connect("tcp://localhost:5555")
    command_block = {
        'command': 'add',
        'kwargs': {
            'tags': args[0],
            'description': args[1],
            'body': body,
        },
    }
    json_message = json.dumps(command_block)
    print("Sending request: %s" % json_message)
    socket.send(json_message)
    message = socket.recv()
    if show:
        print("Received reply %s" % (message))
    return message


def get_request(args, show=True):
    context = zmq.Context()
    socket = context.socket(zmq.REQ)
    socket.connect("tcp://localhost:5555")
    command_block = {
        'command': 'get',
        'kwargs': {
            'id': args[0],
        },
    }
    json_message = json.dumps(command_block)
    print("Sending request: %s" % json_message)
    socket.send(json_message)
    message = socket.recv()
    if show:
        print("Received reply %s" % (message))
    doc = json.loads(message)
    return doc


def search_request(args, show=True):
    context = zmq.Context()
    socket = context.socket(zmq.REQ)
    socket.connect("tcp://localhost:5555")
    command_block = {
        'command': 'search',
        'kwargs': {
            'tag': args[0],
        },
    }
    json_message = json.dumps(command_block)
    print("Sending request: %s" % json_message)
    socket.send(json_message)
    message = socket.recv()
    print("Received reply %s" % (message))
    docs = json.loads(message)
    if show:
        for doc in docs:
            print 'document:'
            print '    id:          {}'.format(doc[0])
            print '    tags:        {}'.format(doc[1])
            print '    description: {}'.format(doc[2])
    return docs


def list_request(args, show=True):
    context = zmq.Context()
    socket = context.socket(zmq.REQ)
    socket.connect("tcp://localhost:5555")
    command_block = {
        'command': 'list',
        'kwargs': {
        },
    }
    json_message = json.dumps(command_block)
    print("Sending request: %s" % json_message)
    socket.send(json_message)
    message = socket.recv()
    print("Received reply %s" % (message))
    docs = json.loads(message)
    if show:
        for doc in docs:
            print 'document:'
            print '    id:          {}'.format(doc[0])
            print '    tags:        {}'.format(doc[1])
            print '    description: {}'.format(doc[2])
    return docs


def usage():
    sys.exit(__doc__)


Command_table = {
    'add': add_request,
    'search': search_request,
    'get': get_request,
    'list': list_request,
}


def main():
    args = sys.argv[1:]
    try:
        opts, args = getopt.getopt(args, 'h', ['help', ])
    except:
        usage()
    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage()
    if len(args) < 1:
        usage()
    command = args[0]
    args = args[1:]
    fn = Command_table.get(command)
    if fn is None:
        sys.stderr.write("command {} not implemented".format(command))
    else:
        fn(args)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
