#!/usr/bin/env python

"""
synopsis:
    Serve documents from a document database.
    Use ZeroMQ as a transport layer.
    Support the following requests:
    - add document.
    - retrieve document by ID.
    - retrieve list of (ID, tags, description) for documents with the
      given tags.
usage:
    python zmq_doc_server01.py [options] <database_file.sqlite>
options:
    --read_only    Only allow reads; no writes.
example:
    python zmq_doc_server01.py my_database_file.sqlite
"""

import sys
import getopt
import zmq
import sqlite3
import json


def add_document(db_connection, kwargs):
    tags = kwargs.get('tags')
    description = kwargs.get('description')
    body = kwargs.get('body')
    body = body.replace('"', '')
    cursor = db_connection.cursor()
    sql = 'select latest_id from config'
    cursor.execute(sql)
    row = cursor.fetchall()
    id = row[0][0]
    new_id = id + 1
    cursor.execute('update config set latest_id={}'.format(new_id))
    sql = 'insert into documents values ({}, "{}", "{}", "{}")'.format(
        new_id, tags, description, body)
    print 'updating -- sql: {}'.format(sql)
    cursor.execute(sql)
    db_connection.commit()
    result = {
        'flag': 'success',
        'id': new_id,
    }
    return result


def search_documents(db_connection, kwargs):
    tag = kwargs.get('tag')
    sql = 'select id, tags,description from documents'
    cursor = db_connection.cursor()
    cursor.execute(sql)
    result_block = []
    for (id, tags, description) in cursor:
        tag_list = tags.split()
        if tag in tag_list:
            result_block.append([id, tags, description])
    return result_block


def get_document(db_connection, kwargs):
    id = kwargs.get('id')
    sql = 'select * from documents where id = {}'.format(id)
    cursor = db_connection.cursor()
    cursor.execute(sql)
    rows = cursor.fetchall()
    if len(rows) < 1:
        result_block = None
    else:
        row = rows[0]
        id, tags, description, body = row
        result_block = {
            'id': id,
            'tags': tags,
            'description': description,
            'body': body,
        }
    return result_block


def list_documents(db_connection, kwargs):
    sql = 'select id, tags,description from documents'
    cursor = db_connection.cursor()
    cursor.execute(sql)
    result_block = [
        [id, tags, description] for (id, tags, description) in cursor]
    return result_block


Command_table = {
    'add': add_document,
    'search': search_documents,
    'get': get_document,
    'list': list_documents,
}


def serve_docs(db_connection, read_only):
    context = zmq.Context()
    socket = context.socket(zmq.REP)
    socket.bind("tcp://*:5555")
    while True:
        #  Wait for next request from client
        json_message = socket.recv()
        print("Received request: %s" % json_message)
        command_block = json.loads(json_message)
        command = command_block.get('command')
        kwargs = command_block.get('kwargs')
        fn = Command_table.get(command)
        if fn is None:
            sys.stderr.write('bad command: "{}"\n'.format(command))
        else:
            result = fn(db_connection, kwargs)
            json_result = json.dumps(result)
            socket.send(json_result)


def usage():
    sys.exit(__doc__)


def main():
    args = sys.argv[1:]
    try:
        opts, args = getopt.getopt(args, 'h', ['help', 'read-only', ])
    except:
        usage()
    read_only = False
    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage()
        elif opt == '--read-only':
            read_only = True
    if len(args) != 1:
        usage()
    db_name = args[0]
    db_connection = sqlite3.connect(db_name)
    serve_docs(db_connection, read_only)

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
