#!/usr/bin/env python

"""
synopsis:
    Create a relational database to hold documents.
    Create tables "config" and "documents".
    Initialize the "latest_id" in "config" to 0.
usage:
    python create_doc_database.py <db_file_name.sqlite>
"""

import sys
import os
import sqlite3

SQL_statement01 = '''
create table config (latest_id integer)
'''
SQL_statement02 = '''
create table documents
(id text, tags text, description text, body text)
'''
SQL_statement03 = '''
insert into config values (0)
'''


def create_tables(db_name):
    con = sqlite3.connect(db_name)
    cursor = con.cursor()
    cursor.execute(SQL_statement01)
    cursor.execute(SQL_statement02)
    cursor.execute(SQL_statement03)
    con.commit()
    con.close()


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        sys.exit(__doc__)
    db_name = args[0]
    if os.path.exists(db_name):
        sys.exit('file {} exists.  aborting.'.format(db_name))
    create_tables(db_name)


if __name__ == '__main__':
    main()
