#!/usr/bin/env python

"""
Synopsis:
    Create sample hdf5 file.
Usage:
    python create_sample_hdf5_file.py <outfilename>
"""

import sys
import numpy as np
import h5py


def create_file(infilename):
    """Create a sample hdf5 file named infilename.
    """
    f = h5py.File(infilename, 'w')
    ds1 = f.create_dataset('dataset1-1', (2,), dtype='i')
    ds1[...] = np.array([x * 2 for x in range(2)], 'i')
    ds2 = f.create_dataset('dataset1-2', (3,), dtype='f')
    ds2[...] = np.array([float(x * 3) for x in range(3)], 'f')
    g1 = f.create_group('subgroup01')
    ds1 = g1.create_dataset('dataset2-1', (4,), dtype='i')
    ds1[...] = np.array([x * 4 for x in range(4)], 'i')
    ds2 = g1.create_dataset('dataset2-2', (5,), dtype='f')
    ds2[...] = np.array([float(x * 5) for x in range(5)], 'f')
    g1 = f.create_group('subgroup02')
    ds1 = g1.create_dataset('dataset3-1', (6,), dtype='i')
    ds1[...] = np.array([x * 6 for x in range(6)], 'i')
    ds2 = g1.create_dataset('dataset3-2', (7,), dtype='f')
    ds2[...] = np.array([float(x * 7) for x in range(7)], 'f')
    g2 = f.create_group('subgroup03')
    ds3 = g2.create_dataset('dataset3-3', (3, 2), dtype='i')
    data1 = []
    for n1 in range(3):
        data1.append([n1 * 4, (n1 * 4) + 1])
    data2 = np.ndarray((3, 2), 'i')
    data2[...] = data1
    ds3[...] = data2
    g3 = g2.create_group('subsubgroup04')
    make_dataset(g3, 'typesampe01', 4, np.int8)
    make_dataset(g3, 'typesampe02', 4, np.int16)
    make_dataset(g3, 'typesampe03', 4, np.int32)
    make_dataset(g3, 'typesampe04', 4, np.int64)
    make_dataset(g3, 'typesampe05', 4, np.float16, float)
    make_dataset(g3, 'typesampe06', 4, np.float32, float)
    make_dataset(g3, 'typesampe07', 4, np.float64, float)
    #make_dataset(g3, 'typesampe08', 4, np.float128, float)
    f.close()


def make_dataset(group, name, size, dtype, pydtype=int):
    ds = group.create_dataset(name, (size,), dtype=dtype)
    data = np.array([pydtype(x * (size + 3)) for x in range(size)], dtype)
    ds[...] = data
    print 'data:', data
    return ds


def main():
    args = sys.argv[1:]
    if len(args) != 1 or args[0].startswith('-'):
        sys.exit(__doc__)
    infilename = args[0]
    create_file(infilename)


if __name__ == '__main__':
    main()
