#!/usr/bin/env python

"""
synopsis:
    Extract a list of items from an hdf5 archive.
    Demonstrate how to do this in several ways.
usage:
    extract_items.py <infilename>
"""


import sys
import h5py
import functools


def accumfn1(accum):
    def fn1(name, obj):
        accum.append((name, obj))
    return fn1


def test1(f):
    accum1 = []
    visitor = accumfn1(accum1)
    f.visititems(visitor)
    return accum1


def accumfn2(accum, name, obj):
    accum.append((name, obj))


def test2(f):
    accum = []
    visitor = functools.partial(accumfn2, accum)
    f.visititems(visitor)
    return accum


def accumfn3(accum, obj):
    accum.append((obj.name, obj))
    #if hasattr(obj, 'itervalues'):
    if isinstance(obj, h5py.Group):
        for childobj in obj.itervalues():
            accumfn3(accum, childobj)


def test3(f):
    accum = []
    accumfn3(accum, f)
    return accum


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        sys.exit(__doc__)
    infilename = args[0]
    h5file = h5py.File(infilename)
    items1 = test1(h5file)
    print items1
    print '-' * 40
    items2 = test2(h5file)
    print items2
    print '-' * 40
    print 'items1 == items2: {}'.format(items1 == items2)
    print '-' * 40
    items3 = test3(h5file)
    print 'items3: {}'. format(items3)
    print '-' * 40
    print 'items2 == items3: {}'.format(items1 == items2)


if __name__ == '__main__':
    main()
