#!/usr/bin/env python

"""
usage: show_dataset_names.py [-h] [-g] [-5] [-p] infilename

Show names of datasets (and groups) in hdf5 XML file

positional arguments:
  infilename       xml input file name (or hdf5 file for --hdf5)

optional arguments:
  -h, --help       show this help message and exit
  -g, --groups     also show group names
  -5, --hdf5       read hdf5 file, not xml
  -p, --full-path  print full path (--hdf5 mode only)
"""

import argparse
import hdf5pylib
import h5py


def show_dataset_names(args):
    infilename = args.infilename
    root = hdf5pylib.parse(infilename, silence=True)
    root_group = root.get_RootGroup()
    do_group(root_group, args, '')


## def do_root_group(root_group, args):
##     indent = ''
##     if args.groups:
##         print '{}group name: "{}"'.format(indent, root_group.get_Name())
##     indent += '    '
##     for dataset in root_group.get_Dataset():
##         # add the dataset
##         do_dataset(dataset, args, indent)
##     for group in root_group.get_Group():
##         do_group(group, args, indent)


def do_group(group, args, indent):
    if args.groups:
        print '{}group name: "{}"'.format(indent, group.get_Name())
    indent += '    '
    for dataset in group.get_Dataset():
        do_dataset(dataset, args, indent)
    for sub_group in group.get_Group():
        do_group(sub_group, args, indent)


def do_dataset(dataset, args, indent):
    name = dataset.get_Name()
    print '{}dataset name: "{}"'.format(indent, name)


def show_dataset_names_hdf5(args):
    infile = h5py.File(args.infilename, 'r')
    do_group_hdf5(infile, args, '')


def do_group_hdf5(group, args, indent):
    for name, item in group.iteritems():
        if args.fullpath:
            name = item.name
        if isinstance(item, h5py.Dataset):
            print '{}dataset: {}'.format(indent, name)
        elif isinstance(item, h5py.Group):
            print '{}group:   {}'.format(indent, name)
            do_group_hdf5(item, args, indent + '    ')


def main():
    parser = argparse.ArgumentParser(
        description='Show names of datasets (and groups) in hdf5 XML file',
        )
    parser.add_argument(
        'infilename', type=str,
        help='xml input file name (or hdf5 file for --hdf5)')
    parser.add_argument(
        '-g', '--groups', dest='groups',
        action='store_true', default=False,
        help='also show group names')
    parser.add_argument(
        '-5', '--hdf5', dest='hdf5',
        action='store_true', default=False,
        help='read hdf5 file, not xml')
    parser.add_argument(
        '-p', '--full-path', dest='fullpath',
        action='store_true', default=False,
        help='print full path (--hdf5 mode only)')
    args = parser.parse_args()
    if args.hdf5:
        show_dataset_names_hdf5(args)
    else:
        show_dataset_names(args)


if __name__ == '__main__':
    # dbg
##     import pdb
##     pdb.set_trace()
    main()
