#!/usr/bin/env python

"""
synopsis:
    Load-balancing broker
    Clients implemented by test03_client.js and test03_client.py.
    Workers implemented in test03_worker.py.
usage:
    $ ./lb_broker01.py & ./lb_worker01.py 1 & ./lb_worker01.py 2 &
    $ /usr/bin/node ./lb_client01.py 3
    $ /usr/bin/node ./lb_client01.js 3
"""

import zmq


FrontAddress = 'tcp://127.0.0.1:12346'
BackAddress = 'tcp://127.0.0.1:12345'


def main():
    """Load balancer main loop."""
    # Prepare context and sockets
    context = zmq.Context.instance()
    frontend = context.socket(zmq.ROUTER)
    frontend.bind(FrontAddress)
    backend = context.socket(zmq.ROUTER)
    backend.bind(BackAddress)
    # Initialize main loop state
    workers = []
    poller = zmq.Poller()
    # Only poll for requests from backend until workers are available
    poller.register(backend, zmq.POLLIN)
    print 'broker waiting'
    while True:
        sockets = dict(poller.poll())
        if backend in sockets:
            # Handle worker activity on the backend
            request = backend.recv_multipart()
            worker, empty, client = request[:3]
            print 'broker received request: {}'.format(request)
            if not workers:
                # Poll for clients now that a worker is available
                poller.register(frontend, zmq.POLLIN)
            workers.append(worker)
            if client != b"READY" and len(request) > 3:
                # If client reply, send rest back to frontend
                empty, reply = request[3:]
                frontend.send_multipart([client, b"", reply])
        if frontend in sockets:
            # Get next client request, route to last-used worker
            data = frontend.recv_multipart()
            print 'broker received request: {}'.format(data)
            client, empty, request = data
            worker = workers.pop(0)
            backend.send_multipart([worker, b"", client, b"", request])
            if not workers:
                # Don't poll clients if no workers are available
                poller.unregister(frontend)
    # Clean up
    backend.close()
    frontend.close()
    context.term()


if __name__ == "__main__":
    main()
