#!/usr/bin/env node

/*
synopsis:
  Load-balancing broker
  Node.js client for the load-balancing broker/worker.
  Send requests asking worker to count elements in an XML document.
  Optionally send a pattern used to filter elements to me counted.
*/

var zmq = require('zmq')
  , frontAddr = 'tcp://127.0.0.1:12346'
  , log = console.log;

var fileList = [
  // [ filename, filter-pattern ]
  ['Data/data02.xml', null, ],
  ['Data/data03.xml', null, ],
  ['Data/data01.xml', 'person', ],
  ['Data/data01.xml', null, ],
  ['Data/missing_file.xml', null, ],
  ['Data/data04.xml', null, ],
  ['Data/data05.xml', null, ],
  ['Data/data06.xml', null, ],
  ['Data/data01.xml', 'person', ],
  ['Data/data01.xml', null, ],
  ['Data/data02.xml', null, ],
  ['Data/data03.xml', null, ],
  ['Data/missing_file.xml', null, ],
  ['Data/data04.xml', null, ],
  ['Data/data05.xml', null, ],
  ['Data/data06.xml', null, ],
  /*
  ['Data/data14.xml', null, ],
  */
];

function clientProcess(ident) {
  var sock = zmq.socket('req')
    , maxNbr = fileList.length
    ;
  sock.identity = "Client-" + ident;
  sock.connect(frontAddr);

  // Start listening for replies.
  var replyNbr = 0;
  sock.on('message', function(payload) {
    replyNbr += 1;
    var args = Array.apply(null, arguments);
    data = JSON.parse(payload);
    log(replyNbr + '. client received  filename: ' + data.filename +
        ' count: ' + data.count);
    if (replyNbr >= maxNbr) {
      log('client finished');
      sock.close();
    }
  });

  // Send requests to the broker.
  for (var idx = 0; idx < fileList.length; ++idx) {
    var filename = fileList[idx][0],
      filter = fileList[idx][1],
      payload = {filename: filename, filter: filter};
    payload = JSON.stringify(payload);
    var lineno = idx + 1;
    log(lineno + '. client sending ' + payload);
    sock.send(payload);
  }
}

function main() {
  var args = process.argv;
  if (args.length != 3) {
    process.stderr.write('usage: test03_client.js <client_number>\n');
    process.exit(1);
  }
  var ident = args[2];
  clientProcess(ident);
}

main();
