#!/usr/bin/env python

"""
synopsis:
    Load-balancing broker
    Python client for the load-balancing broker/worker.
    Send requests asking worker to count elements in an XML document.
    Optionally send a pattern used to filter elements to me counted.
"""

import sys
import zmq
import json


FrontAddr = 'tcp://127.0.0.1:12346'
FileList = [
    # [ filename, filter-pattern ]
    ['Data/data01.xml', 'person', ],
    ['Data/data01.xml', None, ],
    ['Data/data02.xml', None, ],
    ['Data/data03.xml', None, ],
    ['Data/missing_file.xml', None, ],
    ['Data/data04.xml', None, ],
    ['Data/data05.xml', None, ],
    ['Data/data06.xml', None, ],
    ['Data/data01.xml', 'person', ],
    ['Data/data01.xml', None, ],
    ['Data/data02.xml', None, ],
    ['Data/data03.xml', None, ],
    ['Data/missing_file.xml', None, ],
    ['Data/data04.xml', None, ],
    ['Data/data05.xml', None, ],
    ['Data/data06.xml', None, ],
    #['Data/data14.xml', None, ],
]


def client_process(ident):
    """Basic request-reply client using REQ socket."""
    socket = zmq.Context().socket(zmq.REQ)
    ident = 'Client-{}'.format(ident)
    socket.identity = ident.encode("ascii")
    socket.connect(FrontAddr)
    index = 0
    for filename, tag_filter in FileList:
        index += 1
        payload = {'filename': filename, 'filter': tag_filter, }
        payload = json.dumps(payload)
        socket.send(payload)
        print '{}. client sent payload: {}'.format(index, payload)
        payload = socket.recv()
        payload = json.loads(payload)
        filename = payload.get('filename')
        count = payload.get('count')
        print '{}. client {} received filename: {}  count: {}'.format(
            index, ident, filename, count)


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        sys.exit('usage: test03_client.py <client_number>')
    ident = args[0]
    client_process(ident)


if __name__ == "__main__":
    main()
