#!/usr/bin/env python

"""
synopsis:
    Demonstrate the use of asynchronous threads in the asynctools library.
usage:
    asynctools_async.py infilename
notes:
    See: https://pypi.python.org/pypi/asynctools
"""

import sys
from asynctools.threading import Async


@Async
def reader1(infilename):
    length = 0
    with open(infilename, 'r') as infile:
        for line in infile:
            line = line.rstrip()
            print 'line: {}'.format(line)
            length += len(line)
    return length


@Async
def reader2(infilename):
    length = 0
    with open(infilename, 'r') as infile:
        lines = infile.readlines()
        for line in lines:
            line = line.rstrip()
            print 'line: {}'.format(line)
            length += len(line)
    return length


def test(infilename):
    event = reader2(infilename)
    for count in range(20):
        print 'waiting {}'.format(count)
    event.wait()
    for count in range(10, 20):
        print 'continuing {}'.format(count)


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        sys.exit(__doc__)
    infilename = args[0]
    test(infilename)


if __name__ == '__main__':
    main()
