#!/usr/bin/env python

"""
synopsis:
    Demonstrate the use of parallel methods in the asynctools library.
usage:
    ./asynctools_parallel.py
notes:
    See: https://pypi.python.org/pypi/asynctools
"""


import sys
import urllib
import timeit
from asynctools.threading import Parallel


Urls = [
    'http://www.python.org',
    'http://www.davekuhlman.org',
    'https://www.ruby-lang.org/en/',
    'http://www.ruby-doc.org/',
    'https://www.raspberrypi.org/',
    'http://www.ubuntu.com/',
    'https://nodejs.org/en/',
    'https://nodejs.org/en/docs/',
]


def request(url):
    page = urllib.urlopen(url)
    data = page.read()
    return url, data


def test_parallel():
    # Execute
    pll = Parallel(request)
    for url in Urls:
        pll(url)  # Starts a new thread
    # Wait for the results
    results, errors = pll.join()
    for result in results:
        print 'len: {:7d}  url: {}'.format(len(result[1]), result[0])


def test_serial():
    for url in Urls:
        url1, data = request(url)
        print 'len: {:7d}  url: {}'.format(len(data), url1)


def main():
    args = sys.argv[1:]
    if len(args) != 0:
        sys.exit(__doc__)
    test_serial()
    raw_input('Finished serial.  Press Enter.')
    print '-' * 40
    test_parallel()
    raw_input('Finished parallel.  Press Enter.')
    print '-' * 40
    elapsed = timeit.timeit(
        'import asynctools_parallel; asynctools_parallel.test_serial()',
        number=2)
    print 'serial time: {:7.3f}'.format(elapsed)
    print '-' * 40
    elapsed = timeit.timeit(
        'import asynctools_parallel; asynctools_parallel.test_parallel()',
        number=2)
    print 'parallel time: {:7.3f}'.format(elapsed)
    print '-' * 40


if __name__ == '__main__':
    main()
