#!/usr/bin/env python
"""
synopsis:
    Demonstrate various capabilities of BeautifulSoup.
usage:
    python beautifulsoup1.py <infilename> <url>'
examples:
    ./html_beautifulsoup_py3.py ../csv_report.html http://www.python.org
"""

from __future__ import print_function
import sys
import urllib.request
from bs4 import BeautifulSoup


def show(doc):
    #print(doc.prettify())
    print('-' * 50)
    print('title: {}'.format(doc.title))
    print('-' * 50)
    for x in doc.find_all('p'):
        print(x.string)
    print('-' * 50)
    for element in doc.find_all('a'):
        href = element.get('href')
        string = element.string
        print('-----')
        print('element: {}'.format(element))
        print('href: {}  string: "{}"'.format(href, string))
    #return doc


def walk_tree(node):
    print('type: {}'.format(type(node)))
    if hasattr(node, 'name') and node.name is not None:
        print('name: {}  attrs: {}'.format(node.name, node.attrs))
    if hasattr(node, 'children'):
        for child in node.children:
            walk_tree(child)


def main():
    args = sys.argv[1:]
    if len(args) != 2:
        sys.exit(__doc__)
    #
    # Read a document from a file.
    infilename = args[0]
    with open(infilename, 'r') as infile:
        content = infile.read()
    doc = BeautifulSoup(content, 'lxml')
    show(doc)
    #
    # Download a document from the Internet.
    print('=' * 50)
    url = args[1]
    with urllib.request.urlopen(url) as infile:
        content = infile.read()
    doc = BeautifulSoup(content, 'lxml')
    show(doc)
    #
    # Walk the document tree.
    print('=' * 50)
    print('tree walk')
    walk_tree(doc.html)
    #
    # Modify the document tree.  Add a footer.
    tag = doc.new_tag('hr')
    doc.body.append(tag)
    tag = doc.new_tag('p')
    tag.string = 'A simple footer'
    doc.body.append(tag)
    content = doc.prettify()
    with open('tmp1.html', 'w') as outfile:
        outfile.write(content)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
