#!/usr/bin/env python

"""
synopsis:
    XML grep -- search XML docs for xpath pattern.
usage:
    xml_grep.py [options] <pattern> <file1> <file2> ...
options:
    -h, --help      Print this help.
    -v, --verbose   Display extra information.
"""


import sys
import getopt
from lxml import etree


def parsedoc(filename):
    doc = etree.parse(filename)
    root = doc.getroot()
    return doc, root


def grep(pattern, filenames, verbose):
    for filename in filenames:
        doc, root = parsedoc(filename)
        elements = root.xpath(pattern, namespaces=root.nsmap)
        for element in elements:
            if verbose:
                print '{}:{} {} "{}"'.format(
                    filename, element.sourceline, element, element.text)
            else:
                print '{}:{} {}'.format(filename, element.sourceline, element)


def usage():
    sys.exit(__doc__)


def main():
    args = sys.argv[1:]
    try:
        opts, args = getopt.getopt(
            args, 'hv', ['help', 'verbose', ])
    except:
        usage()
    verbose = False
    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage()
        elif opt in ('-v', '--verbose'):
            verbose = True
    if len(args) < 2:
        usage()
    pattern = args[0]
    filenames = args[1:]
    grep(pattern, filenames, verbose)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
