#!/usr/bin/env python

"""
synopsis:
    Demonstate use of Queue.Queue to communicate between threads.
    Use one queue (Work_queue) to send tasks to workers.
    Use another queue (Result_queue) to send results back to the main thread.
    Create output files in the worker threads.
usage:
    python queue301.py
"""


import sys
import os
from Queue import Queue
from threading import Thread
import urllib


Num_worker_threads = 4
Work_queue = Queue()
Result_queue = Queue()
Links = [
    'http://www.python.org',
    'http://www.davekuhlman.org',
    'http://www.tornadoweb.org',
    'http://www.davekuhlman.org/python_multiprocessing_01.html',
    'http://www.ruby-lang.org',
    'https://developer.mozilla.org/en-US/docs/Web/JavaScript',
    'https://nodejs.org/en/',
]
OutputDirectory = 'DataSink'


def worker():
    while True:
        task = Work_queue.get()
        index, url = task
        page = get_page(url)
        outfilename = 'junk{:03d}.html'.format(index)
        outfilename = os.path.join(OutputDirectory, outfilename)
        with open(outfilename, 'w') as outfile:
            outfile.write(page)
        Result_queue.put((url, len(page)))
        Work_queue.task_done()


def get_page(url):
    infile = urllib.urlopen(url)
    data = infile.read()
    infile.close()
    len_data = len(data)
    print('(request) url: {}  len data: {}'.format(url, len_data))
    return data


def test():
    for i in range(Num_worker_threads):
        work_thread = Thread(target=worker)
        work_thread.daemon = True
        work_thread.start()
    for index, url in enumerate(Links):
        Work_queue.put((index, url))
    Work_queue.join()       # block until all tasks are done
    print('-' * 40)
    while not Result_queue.empty():
        result = Result_queue.get()
        url, length = result
        print('length: {1:7d}  url: {0}'.format(url, length))


def main():
    if os.path.exists(OutputDirectory):
        if not os.path.isdir(OutputDirectory):
            sys.exit('\n"{}" exists and is not a directory.\n'.format(
                OutputDirectory))
    else:
        os.mkdir(OutputDirectory)
    test()


if __name__ == '__main__':
    main()
