#!/usr/bin/env python

"""
Synopsis:
    Scan a log file and extract data items using a regular expression.
    For example, from the following line:
        0. 2014.07.22 9:30 PM value1{data10} filler value2{data20} stuff
    extract:
        "data10" and "data20"
Usage:
    python scan_log01.py in_data_file
Examples:
    python scan_log01.py log_data01.txt
"""


import sys
import re


def scan(infile):
    pat1 = re.compile(r'value1{(.*)}.*value2{(.*)}')
    values = []
    for line in infile:
        mo = pat1.search(line)
        if mo is not None:
            items = mo.group(1, 2)
            values.append(items)
    return values


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        sys.exit(__doc__)
    infilename = args[0]
    infile = open(infilename, 'r')
    result = scan(infile)
    infile.close()
    print result


if __name__ == '__main__':
    main()
