#!/usr/bin/env python

"""
synopsis:
    Implement a simple descriptor for exploration.
    This example stores descriptor values in the enclosing object, not
    the descriptor object.
usage:
    python descriptor02.py
"""


class D1(object):
    def __init__(self, name):
        self.name = name

    def __get__(self, obj, objtype):
        print '(__get__) obj: {}  objtype: {}  value: {}'.format(
            obj, objtype, obj.__dict__[self.name])
        return obj.__dict__[self.name]

    def __set__(self, obj, value):
        print '(__set__) obj: {}  value: {}'.format(obj, value)
        obj.__dict__[self.name] = value
        # cannot do the following.  Causes recursion.
        #obj.__setattr__(self.name, value)


class A(object):
    def __init__(self, value):
        self.var1 = value

    var1 = D1('var1')


def main():
    a = A('nil')
    a.var1 = 'abc'
    print '-' * 10
    print 'the current value of a.var1: "{}"'.format(a.var1)
    print 'a.__dict__: {}'.format(a.__dict__)
    return a


if __name__ == '__main__':
    main()
