#!/usr/bin/env python

"""
synopsis:
    Filter stdin to stdout.
Usage:
    python filter01.py <command>
Commands:
    addcomment      Add comment characters at beginning of line.
    remcomment      remove comment characters at beginning of line.
"""


import sys


def addcomment(line):
    return '## %s' % line


def remcomment(line):
    if line.startswith('## '):
        line = line[3:]
    return line


Filter_table = {
    'addcomment': addcomment,
    'remcomment': remcomment,
}


def filter(filter_fn):
    for line in sys.stdin:
        line = filter_fn(line)
        sys.stdout.write(line)


def usage():
    sys.exit(__doc__)


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        usage()
    filter_name = args[0]
    if filter_name not in Filter_table:
        usage()
    filter_fn = Filter_table[filter_name]
    filter(filter_fn)


if __name__ == '__main__':
    main()
